package org.processmining.plugins.timefilter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.SwingConstants;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XLogImpl;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.providedobjects.ProvidedObjectManager;
import org.processmining.plugins.timefilter.ui.GUIUtils;
import org.processmining.plugins.timefilter.ui.LinedChart;
import org.processmining.plugins.timefilter.ui.LinedChart.SORTING;
import org.processmining.plugins.timefilter.ui.UIColors;

import com.fluxicon.slickerbox.components.RoundedPanel;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * This class contains the actual plugin that is used to filter the log
 * 
 * @author Andrea Burattin
 */
public class LogFilter extends JPanel {

	// graphic stuff
	private static final long serialVersionUID = -6544628461530744022L;
	private UIPluginContext context;
	private LinedChart lc;
	private JSlider percentTraces;
	private JSlider percentEvents;
	private JLabel percentTracesTxt;
	private JLabel percentEventsTxt;
	private JButton exportButton;
	private JLabel exportingStatus;
	
	// logic stuff
	private SORTING currentSorting = SORTING.BY_DURATION;
	private XLog log;
	private XLogInfo logInfo;
	private HashMap<SORTING, ArrayList<XTrace>> sortedElements;

	@Plugin(
		name = "Time Based Log Filter",
		parameterLabels = { "A log" },
		returnLabels = { "A log filter panel" },
		returnTypes = { JComponent.class },
		userAccessible = true
	)
	@UITopiaVariant(
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		affiliation = BasicPluginConfiguration.AFFILIATION
	)
	@Visualizer(name = "Time Based Log Filter")
	public JComponent visualize(UIPluginContext context, XLog log) {
		context.log("Preprocessing the log...");
		context.getProgress().setIndeterminate(true);
		
		// check for empty log
		if (log.size() == 0) {
			return GUIUtils.prepareLabel(
					"<html>Error: empty log!<br>At least one trace is required.</html>",
					SwingConstants.CENTER, UIColors.darkGray);
		}
		
		this.context = context;
		this.log = log;
		this.sortedElements = new HashMap<SORTING, ArrayList<XTrace>>();
		preprocessLog();
		initComponents();
		
		context.log("Done!");
		return this;
	}
	
	/**
	 * Updates the texts of the frequencies
	 * 
	 * @param lowerBound the lower index of the array with the selected area
	 * @param upperBound the upper index of the array with the selected area
	 */
	public void updateFrequencies(final int lowerBound, final int upperBound) {
		// update frequencies on separate thread
		new Thread(new Runnable() {
			public void run() {
				// get the log fraction
				List<XTrace> subLog = sortedElements.get(currentSorting).subList(lowerBound, upperBound);

				// trace update
				double traceFraction = (double) subLog.size() / (double) logInfo.getNumberOfTraces();
				percentTraces.setValue((int) (traceFraction * 100));
				percentTracesTxt.setText((int) (traceFraction * 100) + "%");
				
				// events update
				int totEvents = 0;
				for (XTrace t : subLog) {
					totEvents += t.size();
				}
				
				double eventsFraction = (double) totEvents / (double) logInfo.getNumberOfEvents();
				percentEvents.setValue((int) (eventsFraction * 100));
				percentEventsTxt.setText((int) (eventsFraction * 100) + "%");
			}
		}).start();
	}
	
	/**
	 * 
	 */
	private void initComponents() {
		setBackground(new Color(40, 40, 40));
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		
		// Log visualizer
		// ---------------------------------------------------------------------
		lc = new LinedChart(this, sortedElements.get(currentSorting), logInfo, currentSorting);
		RoundedPanel distributionProgressViewer = new RoundedPanel(15, 5, 3);
		distributionProgressViewer.setLayout(new BorderLayout());
		distributionProgressViewer.setBackground(UIColors.lightGray);
		distributionProgressViewer.add(GUIUtils.prepareTitle("Log traces representation"), BorderLayout.NORTH);
		distributionProgressViewer.add(lc, BorderLayout.CENTER);
		
		// Visualization configuration
		// ---------------------------------------------------------------------
		RoundedPanel visualizationConfiguration = new RoundedPanel(15, 5, 3);
		visualizationConfiguration.setLayout(new BorderLayout());
		visualizationConfiguration.setBackground(UIColors.lightGray);
		visualizationConfiguration.setMinimumSize(new Dimension(250, 120));
		visualizationConfiguration.setPreferredSize(new Dimension(250, 120));
		visualizationConfiguration.add(GUIUtils.prepareTitle("Configuration"), BorderLayout.NORTH);
		
		JRadioButton buttonSortDuration = SlickerFactory.instance().createRadioButton("Traces by duration");
		JRadioButton buttonSortStarting = SlickerFactory.instance().createRadioButton("Traces by starting time");
		
		buttonSortDuration.setSelected(true);
		buttonSortDuration.setActionCommand("BY_DURATION");
		buttonSortStarting.setActionCommand("BY_STARTING_EVENT");
		
		ActionListener sortListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				currentSorting = SORTING.valueOf(e.getActionCommand());
				lc.changeSorting(sortedElements.get(currentSorting), currentSorting);
				lc.repaint();
			}
		};
		buttonSortDuration.addActionListener(sortListener);
		buttonSortStarting.addActionListener(sortListener);
		
		ButtonGroup eventColorsGroup = new ButtonGroup();
		eventColorsGroup.add(buttonSortDuration);
		eventColorsGroup.add(buttonSortStarting);
		
		JPanel eventColorPanel = new JPanel();
		eventColorPanel.setLayout(new BoxLayout(eventColorPanel, BoxLayout.Y_AXIS));
		eventColorPanel.setOpaque(false);
		eventColorPanel.add(buttonSortDuration);
		eventColorPanel.add(buttonSortStarting);
		
		visualizationConfiguration.add(eventColorPanel);
		
		// Statistics panel
		// ---------------------------------------------------------------------
		RoundedPanel statsPanel = new RoundedPanel(15, 5, 3);
		statsPanel.setLayout(new GridBagLayout());
		statsPanel.setBackground(UIColors.lightGray);
		statsPanel.setMinimumSize(new Dimension(250, 120));
		statsPanel.setPreferredSize(new Dimension(250, 120));
		
		percentTraces = SlickerFactory.instance().createSlider(SwingConstants.HORIZONTAL);
		percentTraces.setEnabled(false);
		percentTraces.setMinimum(0);
		percentTraces.setMaximum(100);
		percentTraces.setValue(100);
		
		percentEvents = SlickerFactory.instance().createSlider(SwingConstants.HORIZONTAL);
		percentEvents.setEnabled(false);
		percentEvents.setMinimum(0);
		percentEvents.setMaximum(100);
		percentEvents.setValue(100);
		percentEvents.setPaintLabels(true);
		
		percentTracesTxt = GUIUtils.prepareLabel("100%");
		percentEventsTxt = GUIUtils.prepareLabel("100%");
		
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		statsPanel.add(GUIUtils.prepareTitle("Statistics"), c);

		c.gridx = 0; c.gridy = 1;
		statsPanel.add(GUIUtils.prepareLabel("Traces selected:"), c);

		c.gridx = 0; c.gridy = 2;
		c.gridwidth = 2;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		statsPanel.add(percentTraces, c);
		
		c = new GridBagConstraints();
		c.gridx = 2; c.gridy = 2;
		c.gridwidth = 1;
		statsPanel.add(percentTracesTxt, c);

		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 3;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(20, 0, 0, 0);
		statsPanel.add(GUIUtils.prepareLabel("Events selected:"), c);
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 4;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weighty = 1;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.NORTH;
		statsPanel.add(percentEvents, c);
		
		c = new GridBagConstraints();
		c.gridx = 2; c.gridy = 4;
		c.weighty = 1;
		c.anchor = GridBagConstraints.NORTH;
		statsPanel.add(percentEventsTxt, c);
		
		// Export panel
		// ---------------------------------------------------------------------
		RoundedPanel exportPanel = new RoundedPanel(15, 5, 3);
		exportPanel.setLayout(new GridBagLayout());
		exportPanel.setBackground(UIColors.lightGray);
		exportPanel.setMinimumSize(new Dimension(250, 80));
		exportPanel.setPreferredSize(new Dimension(250, 80));
		exportPanel.setLayout(new GridBagLayout());
		
		exportButton = SlickerFactory.instance().createButton("Export selected area");
		exportButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exportSelectedArea();
			}
		});
		
		exportingStatus = GUIUtils.prepareLabel("");
		exportingStatus.setHorizontalAlignment(SwingConstants.CENTER);
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		c.insets = new Insets(10, 10, 5, 10);
		exportPanel.add(exportButton, c);
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 1;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		exportPanel.add(exportingStatus, c);

		
		// Add everything to the main panel
		// ---------------------------------------------------------------------
		JPanel sideContainer = new JPanel();
		sideContainer.setOpaque(false);
		sideContainer.setLayout(new GridBagLayout());
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.VERTICAL;
		sideContainer.add(visualizationConfiguration, c);
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.VERTICAL;
		sideContainer.add(statsPanel, c);
		
		c = new GridBagConstraints();
		c.gridx = 0; c.gridy = 2;
		sideContainer.add(exportPanel, c);
		
		setLayout(new BorderLayout());
		add(sideContainer, BorderLayout.WEST);
		add(distributionProgressViewer, BorderLayout.CENTER);
	}
	
	/**
	 * 
	 */
	private void preprocessLog() {
		
		// prepare log info object
		logInfo = XLogInfoFactory.createLogInfo(log);
		
		// add all the events
		for (XTrace t : log) {
			for (SORTING s : SORTING.values()) {
				ArrayList<XTrace> a = sortedElements.get(s);
				if (a == null) {
					a = new ArrayList<XTrace>();
					sortedElements.put(s, a);
				}
				a.add(t);
			}
		}
		
		// sort the queues
		for (SORTING s : SORTING.values()) {
			Collections.sort(sortedElements.get(s), getComparator(s));
		}
	}
	
	/**
	 * 
	 * @param sorting
	 * @return
	 */
	private Comparator<XTrace> getComparator(SORTING sorting) {
		if (sorting.equals(SORTING.BY_DURATION)) {
			return new Comparator<XTrace>() {
				public int compare(XTrace o1, XTrace o2) {
					long t1f = logInfo.getTraceTimeBoundaries(o1).getStartDate().getTime();
					long t1l = logInfo.getTraceTimeBoundaries(o1).getEndDate().getTime();
					long t2f = logInfo.getTraceTimeBoundaries(o2).getStartDate().getTime();
					long t2l = logInfo.getTraceTimeBoundaries(o2).getEndDate().getTime();
					long t1 = t1l - t1f;
					long t2 = t2l - t2f;
					if (t1 < t2) {
						return +1;
					} else if (t1 > t2) {
						return -1;
					}
					return 0;
				}
			};
		} else if (sorting.equals(SORTING.BY_STARTING_EVENT)) {
			return new Comparator<XTrace>() {
				public int compare(XTrace o1, XTrace o2) {
					long t1 = logInfo.getTraceTimeBoundaries(o1).getStartDate().getTime();
					long t2 = logInfo.getTraceTimeBoundaries(o2).getStartDate().getTime();
					if (t1 < t2) {
						return -1;
					} else if (t1 > t2) {
						return +1;
					}
					return 0;
				}
			};
		}
		return null;
	}
	
	/**
	 * 
	 */
	private void exportSelectedArea() {
		exportingStatus.setText("Exporting to workspace...");
		
		XLog subXLog = new XLogImpl((XAttributeMap) log.getAttributes().clone());
		List<XTrace> subLog = sortedElements.get(currentSorting).subList(lc.getLowerBound(), lc.getUpperBound());
		for (XTrace t : subLog) {
			subXLog.add((XTrace) t.clone());
		}
		String name = log.getAttributes().get("concept:name").toString() + " (subset created at " + new Date().toString() + ")";
		ProvidedObjectManager pom = context.getProvidedObjectManager();
		pom.createProvidedObject(name, subXLog, XLog.class, context);
		context.getGlobalContext().getResourceManager().getResourceForInstance(subXLog).setFavorite(true);
		
		exportingStatus.setText("Exported to workspace!");
		
		// reset after 5 seconds...
		new Thread(new Runnable() {
			public void run() {
				try { Thread.sleep(5000); } catch (InterruptedException e) { }
				exportingStatus.setText("");
			}
		}).start();
	}
}
